######################################################################
#                                                                    #
# -- Makefile script for the Autotuner project - hhs, GPULab 2010 -- #
#                                                                    #
######################################################################

CUDA_INSTALL_PATH ?= /usr/local/cuda
CUDA_SDK_INSTALL_PATH = $(CUDA_INSTALL_PATH)/sdk

ifneq ($(DARWIN),)
    CUDA_SDK_INSTALL_PATH = /Developer/GPU\ Computing
endif

ifdef cuda-install
	CUDA_INSTALL_PATH := $(cuda-install)
endif

# detect OS
OSUPPER = $(shell uname -s 2>/dev/null | tr [:lower:] [:upper:])
OSLOWER = $(shell uname -s 2>/dev/null | tr [:upper:] [:lower:])

# 'linux' is output for Linux system, 'darwin' for OS X
DARWIN = $(strip $(findstring DARWIN, $(OSUPPER)))
ifneq ($(DARWIN),)
   SNOWLEOPARD = $(strip $(findstring 10.6, $(shell egrep "<string>10\.6" /System/Library/CoreServices/SystemVersion.plist)))
endif

# detect 32-bit or 64-bit platform
HP_64 = $(shell uname -m | grep 64)
OSARCH= $(shell uname -m)
SRCDIR     ?= 
ROOTDIR    ?= ..
LIBDIR     := $(ROOTDIR)/../lib

# Compilers
NVCC       := $(CUDA_INSTALL_PATH)/bin/nvcc
CXX        := g++
CC         := gcc
LD         := g++ -fPIC

INCLUDES   ?= -I$(ROOTDIR)

INCLUDES  += -I/usr/include
INCLUDES  += -I$(CUDA_SDK_INSTALL_PATH)/C/common/inc -I$(CUDA_SDK_INSTALL_PATH)/C/shared/inc
#INCLUDES  += -I$(ROOTDIR)/../cuda_libraries/magma_1.0.0-rc5/include

# Warning flags
NVCCWARN_FLAGS := \
	-w \

CXXWARN_FLAGS := \
	-W -Wall \
	-Wimplicit \
	-Wswitch \
	-Wformat \
	-Wchar-subscripts \
	-Wparentheses \
	-Wmultichar \
	-Wtrigraphs \
	-Wpointer-arith \
	-Wcast-align \
	-Wreturn-type \
	-Wno-unused-function \
	$(SPACE)

CWARN_FLAGS := $(CXXWARN_FLAGS) \
	-Wstrict-prototypes \
	-Wmissing-prototypes \
	-Wmissing-declarations \
	-Wnested-externs \
	-Wmain \

# architecture flag for nvcc and gcc compilers build
#CUBIN_ARCH_FLAG :=
CXX_ARCH_FLAGS  := $(ATUNE_FLAGS)
NVCC_ARCH_FLAGS  := -Xptxas=-v -arch sm_$(ARCH) $(ATUNE_FLAGS)
LIB_ARCH        := $(OSARCH)
# In order to know the architecture in host code 
# we define the following card generation flags
ifeq ($(strip $(MAJOR)),1)
	CXX_ARCH_FLAGS  += -D_TESLA 
	NVCC_ARCH_FLAGS  += -D_TESLA 
endif
ifeq ($(strip $(MAJOR)),2)
	CXX_ARCH_FLAGS  += -D_FERMI 
	NVCC_ARCH_FLAGS  += -D_FERMI 
endif

# Determining the necessary Cross-Compilation Flags
# 32-bit OS, but we target 64-bit cross compilation
ifeq ($(x86_64),1) 
    NVCCFLAGS       += -m64
    LIB_ARCH         = x86_64
    CUDPPLIB_SUFFIX  = x86_64
    ifneq ($(DARWIN),)
         CXX_ARCH_FLAGS += -arch x86_64
    else
         CXX_ARCH_FLAGS += -m64
    endif
else 
# 64-bit OS, and we target 32-bit cross compilation
    ifeq ($(i386),1)
        NVCCFLAGS       += -m32
        LIB_ARCH         = i386
        CUDPPLIB_SUFFIX  = i386
        ifneq ($(DARWIN),)
             CXX_ARCH_FLAGS += -arch i386
        else
             CXX_ARCH_FLAGS += -m32
        endif
    else 
        ifeq "$(strip $(HP_64))" ""
            LIB_ARCH        = i386
            CUDPPLIB_SUFFIX = i386
            NVCCFLAGS      += -m32
            ifneq ($(DARWIN),)
               CXX_ARCH_FLAGS += -arch i386
            else
               CXX_ARCH_FLAGS += -m32
            endif
        else
            LIB_ARCH        = x86_64
            CUDPPLIB_SUFFIX = x86_64
            NVCCFLAGS      += -m64
            ifneq ($(DARWIN),)
               CXX_ARCH_FLAGS += -arch x86_64
            else
               CXX_ARCH_FLAGS += -m64
            endif
        endif
    endif
endif

NVCCFLAGS  += $(NVCCWARN_FLAGS) $(NVCC_ARCH_FLAGS)
CXXFLAGS  += $(CXXWARN_FLAGS) $(CXX_ARCH_FLAGS)
CFLAGS    += $(CWARN_FLAGS) $(CXX_ARCH_FLAGS)
LDFLAGS += $(CXX_ARCH_FLAGS)

# This option for Mac allows CUDA applications to work without requiring to set DYLD_LIBRARY_PATH
ifneq ($(DARWIN),)
   LDFLAGS += -Xlinker -rpath $(CUDA_INSTALL_PATH)/lib
endif

# Common flags
COMMONFLAGS += $(INCLUDES)

ifeq ($(DARWIN),)
   COMMONFLAGS += -DUNIX
endif

# Debug/release configuration
ifeq ($(dbg),1)
	COMMONFLAGS += -g
	NVCCFLAGS   += -D_DEBUG
	CXXFLAGS    += -D_DEBUG
	CFLAGS      += -D_DEBUG
else 
	COMMONFLAGS += -O3
	NVCCFLAGS   += --compiler-options -fno-strict-aliasing
	CXXFLAGS    += -fno-strict-aliasing
	CFLAGS      += -fno-strict-aliasing
endif
#NVCCFLAGS   += --ptxas-options=-v -D_DEBUG

# architecture flag for cubin build
CUBIN_ARCH_FLAG :=

# Includes
INCLUDES  += -I. -I$(CUDA_INSTALL_PATH)/include

# Libs
ifneq ($(DARWIN),)
    LIBS       := -L$(CUDA_INSTALL_PATH)/lib
else
  ifeq "$(strip $(HP_64))" ""
    ifeq ($(x86_64),1)
       LIBS       := -L$(CUDA_INSTALL_PATH)/lib64
    else
       LIBS       := -L$(CUDA_INSTALL_PATH)/lib
    endif
  else
    ifeq ($(i386),1)
       LIBS       := -L$(CUDA_INSTALL_PATH)/lib
    else
       LIBS       := -L$(CUDA_INSTALL_PATH)/lib64
    endif
  endif
endif

LIBS += -L$(CUDA_SDK_INSTALL_PATH)/C/lib

# libraries to be linked with
ifneq ($(DARWIN),)
    LIBS += -lcublas -lcblas -llapack -lcuda -lcudart -lcutil -lpthread
else
    LIBS += -lcublas -lblas -llapack -lcuda -lcudart -lcutil -lpthread
endif

#LIBS += -L$(HOME)/cuda_libraries/magma_1.0.0-rc5/lib -lmagma -lmagmablas

# -lgetnb


# check if verbose 
ifeq ($(verbose), 1)
	VERBOSE :=
else
	VERBOSE := @
endif

ifeq ($(fastmath), 1)
	NVCCFLAGS += -use_fast_math
endif

ifeq ($(nvccverbose), 1)
        NVCCFLAGS += -v
endif

ifeq ($(keep), 1)
	NVCCFLAGS += -keep
	NVCC_KEEP_CLEAN := *.i* *.cubin *.cu.c *.cudafe* *.fatbin.c *.ptx
endif

ifdef maxregisters
	NVCCFLAGS += -maxrregcount $(maxregisters)
endif

# Add cudacc flags
NVCCFLAGS += $(CUDACCFLAGS)

# Add common flags
NVCCFLAGS += $(COMMONFLAGS)
CXXFLAGS  += $(COMMONFLAGS)
CFLAGS    += $(COMMONFLAGS)

ifeq ($(nvcc_warn_verbose),1)
	NVCCFLAGS += $(addprefix --compiler-options ,$(CXXWARN_FLAGS)) 
	NVCCFLAGS += --compiler-options -fno-strict-aliasing
endif

# Meta rules to compile targets
$(TARGET): $(OBJS) $(KERNELS) $(DEPS)
	$(LD) $(LDFLAGS) $(OBJS) -o $(TARGET) $(KERNELS) $(LIBS) 

$(LIBTARGET): $(OBJS) $(KERNELS) $(DEPS)
	rm -f $(LIBTARGET)
	$(AR) rcs $(LIBTARGET) $(OBJS) $(KERNELS)

s%.o: s%.c
	$(CC) $(CFLAGS) -D_FLOAT -c $<

s%.o: s%.cpp
	$(CXX) $(CXXFLAGS) -D_FLOAT -c $<

s%.o: s%.cu
	$(NVCC) $(NVCCFLAGS) -D_FLOAT -c $<

is%.o: is%.c
	$(CC) $(CFLAGS) -D_FLOAT -c $<

is%.o: is%.cpp
	$(CXX) $(CXXFLAGS) -D_FLOAT -c $<

is%.o: is%.cu
	$(NVCC) $(NVCCFLAGS) -D_FLOAT -c $<

d%.o: d%.c
	$(CC) $(CFLAGS) -D_DOUBLE -c $<

d%.o: d%.cpp
	$(CXX) $(CXXFLAGS) -D_DOUBLE -c $<

d%.o: d%.cu
	$(NVCC) $(NVCCFLAGS) -D_DOUBLE -c $<

id%.o: id%.c
	$(CC) $(CFLAGS) -D_DOUBLE -c $<

id%.o: id%.cpp
	$(CXX) $(CXXFLAGS) -D_DOUBLE -c $<

id%.o: id%.cu
	$(NVCC) $(NVCCFLAGS) -D_DOUBLE -c $<

.PHONY : clean run

clean:
	/bin/rm -f $(TARGET) $(LIBTARGET) $(OBJS) $(KERNELS)

run:
	./$(TARGET) 


