#pragma once

#define max(a,b) (a > b ? a : b)

#if defined (__CUDA_ARCH__) && __CUDA_ARCH__ <= 100
#error Cuda capability less than 1.1 is not supported!
#endif

#if defined (__CUDA_ARCH__) && __CUDA_ARCH__ < 200

//
// If capability is less than 2.0 we need to make our own atomicAdd function for floats.
//

#define atomicAdd atomicFloatAdd

__device__ void atomicFloatAdd(float* address, float val) 
{ 
	float old = *address, assumed; 
	do { 
		assumed = old; 
		old = __int_as_float(atomicCAS((unsigned int*)address, 
									   __float_as_int(assumed), 
									   __float_as_int(val + assumed))); 
	} while (assumed != old);
}

#endif

#ifdef _FLOAT

//
// For capability 2.x we need to make our own atomicMax and atomixImax functions for floats.
//
inline __device__ void atomicMax(float* address, float val) 
{ 
	unsigned int* address_as_u = (unsigned int*) address; 
	unsigned int old = *address_as_u, assumed; 
	do
	{ 
		assumed = old; 
		old = atomicCAS(address_as_u, assumed, __float_as_int( max(val, __int_as_float(assumed) ))); 
	} while (assumed != old); 
}

inline __device__ void atomicImax(const float *address, float val, int *iaddress, int idx) 
{
	int old = *iaddress, assumed; 
	do
	{
		assumed = old;
		float themax = address[assumed];
		if (val > themax)
		{
			old = atomicCAS(iaddress, assumed, idx); 
		}
	} while (assumed != old); 
}

#endif

#ifdef _DOUBLE
#define atomicAdd atomicDoubleAdd

//
// For capability 2.x we need to make our own atomicAdd and atomixMax function for doubles
//
inline __device__ void atomicDoubleAdd(double* address, double val) 
{ 
	unsigned long long int* address_as_ull = (unsigned long long int*)address; 
	unsigned long long int old = *address_as_ull, assumed; 
	do 
	{ 
		assumed = old; 
		old = atomicCAS(address_as_ull, assumed, __double_as_longlong(val + __longlong_as_double(assumed))); 
	} while (assumed != old); 
}
inline __device__ void atomicDoubleMax(double* address, double val) 
{ 
	unsigned long long int* address_as_ull = (unsigned long long int*)address; 
	unsigned long long int old = *address_as_ull, assumed; 
	do 
	{ 
		assumed = old; 
		old = atomicCAS(address_as_ull, assumed, __double_as_longlong(max(val, __longlong_as_double(assumed)))); 
	} while (assumed != old); 
}
/*
inline __device__ double atomicDoubleAdd(double* address, double val) 
{ 
	double old = *address, assumed; 
	do { 
		assumed = old; 
		old = __longlong_as_double( atomicCAS((unsigned long long int*)address, 
											  __double_as_longlong(assumed), 
											  __double_as_longlong(val + assumed))); 
	} while (assumed != old);
}
*/
#endif

