//
// Atune - Auto-Tuning of Numerical Applications for GPUs.
//
// - Version 1.0 (April 2011)
//
// - Hans Henrik Brandenborg Sørensen, hhs@imm.dtu.dk
// - GPUlab, DTU Informatics
// - Technical University of Denmark
//
#define m_max 1
#define n_max 177827942
#define m_grid 1
#define n_grid 264
#define m_step (log10(m_max)/m_grid)
#define n_step (log10(n_max)/n_grid)
#define m_idx ((int)(log10(m)/m_step))
#define n_idx ((int)(log10(n)/n_step))

int besttable_isamax[] = { 
6, 1, 5, 7, 7, 8, 3, 7, 3, 2, 9, 3, 3, 2, 5, 2, 2, 1, 51, 4, 4, 4, 4, 4, 4, 8, 3, 8, 4, 14, 3, 3, 19, 9, 9, 5, 5, 5, 8, 1, 1, 5, 1, 1, 13, 5, 1, 5, 10, 2, 2, 20, 21, 20, 2, 19, 21, 2, 3, 2, 3, 3, 3, 3, 3, 2, 2, 4, 12, 3, 4, 3, 12, 2, 3, 2, 11, 3, 2, 2, 3, 15, 2, 3, 12, 3, 3, 3, 2, 2, 12, 3, 2, 2, 2, 2, 3, 11, 3, 11, 3, 11, 13, 3, 13, 3, 13, 28, 4, 15, 3, 4, 29, 4, 13, 15, 15, 13, 13, 13, 13, 15, 13, 15, 15, 16, 13, 16, 13, 15, 22, 15, 15, 16, 15, 29, 23, 16, 16, 45, 45, 16, 16, 16, 34, 34, 23, 30, 30, 30, 17, 42, 30, 46, 48, 37, 30, 38, 24, 17, 58, 18, 47, 52, 62, 25, 31, 47, 61, 32, 65, 65, 35, 53, 39, 47, 62, 39, 25, 32, 68, 66, 32, 43, 62, 36, 39, 66, 40, 44, 19, 54, 49, 54, 57, 59, 41, 69, 55, 50, 50, 50, 60, 26, 64, 63, 67, 25, 25, 26, 26, 26, 56, 27, 20, 21, 25, 25, 25, 20, 25, 32, 25, 25, 25, 25, 25, 25, 33, 25, 25, 25, 20, 25, 25, 25, 25, 25, 25, 25, 26, 25, 26, 25, 25, 25, 25, 25, 25, 25, 25, 25, 25, 25, 25, 25, 25, 25, 25, 25, 25, 25, 25, 25, 
};

int atune_isamax(isamax_arguments)
{
   int bestkernel = besttable_isamax[n_idx];
   switch (bestkernel)
   {
   case 1:
      isamax_v1<1, 16, 1, 1>(x_d, y_d, n);
      break;
   case 2:
      isamax_v1<1, 16, 1, 2>(x_d, y_d, n);
      break;
   case 3:
      isamax_v1<1, 16, 1, 4>(x_d, y_d, n);
      break;
   case 4:
      isamax_v1<1, 16, 1, 8>(x_d, y_d, n);
      break;
   case 5:
      isamax_v1<1, 16, 1, 16>(x_d, y_d, n);
      break;
   case 6:
      isamax_v1<1, 16, 1, 32>(x_d, y_d, n);
      break;
   case 7:
      isamax_v1<1, 16, 1, 64>(x_d, y_d, n);
      break;
   case 8:
      isamax_v1<1, 16, 1, 128>(x_d, y_d, n);
      break;
   case 9:
      isamax_v1<1, 16, 1, 256>(x_d, y_d, n);
      break;
   case 10:
      isamax_v1<1, 32, 1, 1>(x_d, y_d, n);
      break;
   case 11:
      isamax_v1<1, 32, 1, 2>(x_d, y_d, n);
      break;
   case 12:
      isamax_v1<1, 32, 1, 4>(x_d, y_d, n);
      break;
   case 13:
      isamax_v1<1, 32, 1, 8>(x_d, y_d, n);
      break;
   case 14:
      isamax_v1<1, 32, 1, 256>(x_d, y_d, n);
      break;
   case 15:
      isamax_v1<1, 64, 1, 4>(x_d, y_d, n);
      break;
   case 16:
      isamax_v1<1, 64, 1, 8>(x_d, y_d, n);
      break;
   case 17:
      isamax_v1<1, 64, 1, 16>(x_d, y_d, n);
      break;
   case 18:
      isamax_v1<1, 64, 1, 32>(x_d, y_d, n);
      break;
   case 19:
      isamax_v1<1, 64, 1, 64>(x_d, y_d, n);
      break;
   case 20:
      isamax_v1<1, 64, 1, 128>(x_d, y_d, n);
      break;
   case 21:
      isamax_v1<1, 64, 1, 256>(x_d, y_d, n);
      break;
   case 22:
      isamax_v1<1, 96, 1, 4>(x_d, y_d, n);
      break;
   case 23:
      isamax_v1<1, 96, 1, 8>(x_d, y_d, n);
      break;
   case 24:
      isamax_v1<1, 96, 1, 16>(x_d, y_d, n);
      break;
   case 25:
      isamax_v1<1, 96, 1, 32>(x_d, y_d, n);
      break;
   case 26:
      isamax_v1<1, 96, 1, 64>(x_d, y_d, n);
      break;
   case 27:
      isamax_v1<1, 96, 1, 128>(x_d, y_d, n);
      break;
   case 28:
      isamax_v1<1, 128, 1, 2>(x_d, y_d, n);
      break;
   case 29:
      isamax_v1<1, 128, 1, 4>(x_d, y_d, n);
      break;
   case 30:
      isamax_v1<1, 128, 1, 8>(x_d, y_d, n);
      break;
   case 31:
      isamax_v1<1, 128, 1, 16>(x_d, y_d, n);
      break;
   case 32:
      isamax_v1<1, 128, 1, 32>(x_d, y_d, n);
      break;
   case 33:
      isamax_v1<1, 128, 1, 64>(x_d, y_d, n);
      break;
   case 34:
      isamax_v1<1, 160, 1, 8>(x_d, y_d, n);
      break;
   case 35:
      isamax_v1<1, 160, 1, 16>(x_d, y_d, n);
      break;
   case 36:
      isamax_v1<1, 160, 1, 64>(x_d, y_d, n);
      break;
   case 37:
      isamax_v1<1, 192, 1, 8>(x_d, y_d, n);
      break;
   case 38:
      isamax_v1<1, 192, 1, 16>(x_d, y_d, n);
      break;
   case 39:
      isamax_v1<1, 192, 1, 32>(x_d, y_d, n);
      break;
   case 40:
      isamax_v1<1, 192, 1, 64>(x_d, y_d, n);
      break;
   case 41:
      isamax_v1<1, 192, 1, 128>(x_d, y_d, n);
      break;
   case 42:
      isamax_v1<1, 224, 1, 8>(x_d, y_d, n);
      break;
   case 43:
      isamax_v1<1, 224, 1, 32>(x_d, y_d, n);
      break;
   case 44:
      isamax_v1<1, 224, 1, 64>(x_d, y_d, n);
      break;
   case 45:
      isamax_v1<1, 256, 1, 4>(x_d, y_d, n);
      break;
   case 46:
      isamax_v1<1, 256, 1, 8>(x_d, y_d, n);
      break;
   case 47:
      isamax_v1<1, 256, 1, 16>(x_d, y_d, n);
      break;
   case 48:
      isamax_v1<1, 256, 1, 32>(x_d, y_d, n);
      break;
   case 49:
      isamax_v1<1, 256, 1, 64>(x_d, y_d, n);
      break;
   case 50:
      isamax_v1<1, 256, 1, 128>(x_d, y_d, n);
      break;
   case 51:
      isamax_v1<1, 288, 1, 2>(x_d, y_d, n);
      break;
   case 52:
      isamax_v1<1, 288, 1, 16>(x_d, y_d, n);
      break;
   case 53:
      isamax_v1<1, 288, 1, 32>(x_d, y_d, n);
      break;
   case 54:
      isamax_v1<1, 288, 1, 64>(x_d, y_d, n);
      break;
   case 55:
      isamax_v1<1, 288, 1, 128>(x_d, y_d, n);
      break;
   case 56:
      isamax_v1<1, 288, 1, 256>(x_d, y_d, n);
      break;
   case 57:
      isamax_v1<1, 320, 1, 64>(x_d, y_d, n);
      break;
   case 58:
      isamax_v1<1, 352, 1, 32>(x_d, y_d, n);
      break;
   case 59:
      isamax_v1<1, 352, 1, 64>(x_d, y_d, n);
      break;
   case 60:
      isamax_v1<1, 352, 1, 128>(x_d, y_d, n);
      break;
   case 61:
      isamax_v1<1, 384, 1, 16>(x_d, y_d, n);
      break;
   case 62:
      isamax_v1<1, 384, 1, 32>(x_d, y_d, n);
      break;
   case 63:
      isamax_v1<1, 416, 1, 64>(x_d, y_d, n);
      break;
   case 64:
      isamax_v1<1, 416, 1, 128>(x_d, y_d, n);
      break;
   case 65:
      isamax_v1<1, 448, 1, 16>(x_d, y_d, n);
      break;
   case 66:
      isamax_v1<1, 480, 1, 32>(x_d, y_d, n);
      break;
   case 67:
      isamax_v1<1, 480, 1, 128>(x_d, y_d, n);
      break;
   case 68:
      isamax_v1<1, 512, 1, 32>(x_d, y_d, n);
      break;
   case 69:
      isamax_v1<1, 512, 1, 64>(x_d, y_d, n);
      break;
   default:
      printf("\nisamax.h : bestkernelSwitch : problem size not supported (n = %d, bestkernel = %d).\n", n, bestkernel);
      exit(EXIT_FAILURE);
   }
   return bestkernel;
}
