//
// Atune - Auto-Tuning of Numerical Applications for GPUs.
//
// - Version 1.0 (April 2011)
//
// - Hans Henrik Brandenborg Sørensen, hhs@imm.dtu.dk
// - GPUlab, DTU Informatics
// - Technical University of Denmark
//
#define m_max 1
#define n_max 177827942
#define m_grid 1
#define n_grid 264
#define m_step (log10(m_max)/m_grid)
#define n_step (log10(n_max)/n_grid)
#define m_idx ((int)(log10(m)/m_step))
#define n_idx ((int)(log10(n)/n_step))

int besttable_saxpy[] = { 
1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 38, 7, 2, 7, 6, 12, 2, 10, 2, 10, 10, 10, 9, 12, 10, 11, 22, 2, 14, 9, 12, 2, 12, 8, 12, 10, 12, 9, 11, 14, 24, 3, 26, 13, 8, 14, 2, 15, 6, 4, 38, 13, 2, 25, 6, 8, 2, 13, 4, 8, 5, 6, 2, 5, 2, 2, 13, 8, 22, 22, 2, 23, 2, 8, 22, 22, 2, 13, 5, 9, 5, 8, 22, 14, 13, 22, 5, 22, 8, 8, 13, 22, 13, 8, 8, 22, 8, 8, 13, 8, 9, 13, 22, 13, 22, 13, 13, 13, 13, 13, 13, 13, 22, 13, 22, 13, 22, 22, 20, 35, 22, 16, 28, 23, 18, 37, 32, 28, 24, 17, 17, 21, 21, 29, 30, 33, 33, 31, 31, 19, 34, 46, 48, 50, 27, 43, 45, 41, 39, 22, 50, 16, 42, 22, 22, 22, 22, 39, 39, 22, 36, 39, 22, 39, 39, 39, 39, 39, 39, 39, 22, 39, 22, 39, 39, 39, 39, 39, 22, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 39, 40, 40, 40, 47, 49, 51, 47, 47, 51, 51, 49, 51, 51, 47, 47, 47, 51, 49, 51, 44, 44, 44, 52, 52, 52, 52, 52, 52, 52, 53, 53, 53, 53, 
};

int atune_saxpy(saxpy_arguments)
{
   int bestkernel = besttable_saxpy[n_idx];
   switch (bestkernel)
   {
   case 1:
      cublasSaxpy(n, alpha, x_d, 1, y_d, 1);
      break;
   case 2:
      saxpy_v1<1, 16, 1, 1>(alpha, x_d, y_d, n);
      break;
   case 3:
      saxpy_v1<1, 16, 1, 2>(alpha, x_d, y_d, n);
      break;
   case 4:
      saxpy_v1<1, 16, 1, 4>(alpha, x_d, y_d, n);
      break;
   case 5:
      saxpy_v1<1, 32, 1, 1>(alpha, x_d, y_d, n);
      break;
   case 6:
      saxpy_v1<1, 32, 1, 2>(alpha, x_d, y_d, n);
      break;
   case 7:
      saxpy_v1<1, 32, 1, 4>(alpha, x_d, y_d, n);
      break;
   case 8:
      saxpy_v1<1, 64, 1, 1>(alpha, x_d, y_d, n);
      break;
   case 9:
      saxpy_v1<1, 64, 1, 2>(alpha, x_d, y_d, n);
      break;
   case 10:
      saxpy_v1<1, 64, 1, 4>(alpha, x_d, y_d, n);
      break;
   case 11:
      saxpy_v1<1, 64, 1, 8>(alpha, x_d, y_d, n);
      break;
   case 12:
      saxpy_v1<1, 64, 1, 16>(alpha, x_d, y_d, n);
      break;
   case 13:
      saxpy_v1<1, 128, 1, 1>(alpha, x_d, y_d, n);
      break;
   case 14:
      saxpy_v1<1, 128, 1, 2>(alpha, x_d, y_d, n);
      break;
   case 15:
      saxpy_v1<1, 128, 1, 8>(alpha, x_d, y_d, n);
      break;
   case 16:
      saxpy_v1<1, 160, 1, 2>(alpha, x_d, y_d, n);
      break;
   case 17:
      saxpy_v1<1, 160, 1, 4>(alpha, x_d, y_d, n);
      break;
   case 18:
      saxpy_v1<1, 192, 1, 2>(alpha, x_d, y_d, n);
      break;
   case 19:
      saxpy_v1<1, 192, 1, 8>(alpha, x_d, y_d, n);
      break;
   case 20:
      saxpy_v1<1, 224, 1, 1>(alpha, x_d, y_d, n);
      break;
   case 21:
      saxpy_v1<1, 224, 1, 4>(alpha, x_d, y_d, n);
      break;
   case 22:
      saxpy_v1<1, 256, 1, 1>(alpha, x_d, y_d, n);
      break;
   case 23:
      saxpy_v1<1, 256, 1, 2>(alpha, x_d, y_d, n);
      break;
   case 24:
      saxpy_v1<1, 256, 1, 4>(alpha, x_d, y_d, n);
      break;
   case 25:
      saxpy_v1<1, 256, 1, 8>(alpha, x_d, y_d, n);
      break;
   case 26:
      saxpy_v1<1, 256, 1, 16>(alpha, x_d, y_d, n);
      break;
   case 27:
      saxpy_v1<1, 288, 1, 8>(alpha, x_d, y_d, n);
      break;
   case 28:
      saxpy_v1<1, 320, 1, 2>(alpha, x_d, y_d, n);
      break;
   case 29:
      saxpy_v1<1, 320, 1, 4>(alpha, x_d, y_d, n);
      break;
   case 30:
      saxpy_v1<1, 352, 1, 4>(alpha, x_d, y_d, n);
      break;
   case 31:
      saxpy_v1<1, 384, 1, 8>(alpha, x_d, y_d, n);
      break;
   case 32:
      saxpy_v1<1, 416, 1, 2>(alpha, x_d, y_d, n);
      break;
   case 33:
      saxpy_v1<1, 416, 1, 4>(alpha, x_d, y_d, n);
      break;
   case 34:
      saxpy_v1<1, 480, 1, 8>(alpha, x_d, y_d, n);
      break;
   case 35:
      saxpy_v1<1, 512, 1, 1>(alpha, x_d, y_d, n);
      break;
   case 36:
      saxpy_v1<1, 512, 1, 2>(alpha, x_d, y_d, n);
      break;
   case 37:
      saxpy_v1<1, 512, 1, 4>(alpha, x_d, y_d, n);
      break;
   case 38:
      saxpy_v1<1, 512, 1, 16>(alpha, x_d, y_d, n);
      break;
   case 39:
      saxpy_v2<1, 128, 1, 2>(alpha, x_d, y_d, n);
      break;
   case 40:
      saxpy_v2<1, 160, 1, 2>(alpha, x_d, y_d, n);
      break;
   case 41:
      saxpy_v2<1, 160, 1, 16>(alpha, x_d, y_d, n);
      break;
   case 42:
      saxpy_v2<1, 192, 1, 2>(alpha, x_d, y_d, n);
      break;
   case 43:
      saxpy_v2<1, 224, 1, 8>(alpha, x_d, y_d, n);
      break;
   case 44:
      saxpy_v2<1, 320, 1, 4>(alpha, x_d, y_d, n);
      break;
   case 45:
      saxpy_v2<1, 320, 1, 16>(alpha, x_d, y_d, n);
      break;
   case 46:
      saxpy_v2<1, 416, 1, 8>(alpha, x_d, y_d, n);
      break;
   case 47:
      saxpy_v2<1, 448, 1, 2>(alpha, x_d, y_d, n);
      break;
   case 48:
      saxpy_v2<1, 448, 1, 8>(alpha, x_d, y_d, n);
      break;
   case 49:
      saxpy_v2<1, 480, 1, 2>(alpha, x_d, y_d, n);
      break;
   case 50:
      saxpy_v2<1, 480, 1, 4>(alpha, x_d, y_d, n);
      break;
   case 51:
      saxpy_v2<1, 512, 1, 2>(alpha, x_d, y_d, n);
      break;
   case 52:
      saxpy_v2<1, 512, 1, 4>(alpha, x_d, y_d, n);
      break;
   case 53:
      saxpy_v2<1, 1024, 1, 8>(alpha, x_d, y_d, n);
      break;
   default:
      printf("\nsaxpy.h : bestkernelSwitch : problem size not supported (n = %d, bestkernel = %d).\n", n, bestkernel);
      exit(EXIT_FAILURE);
   }
   return bestkernel;
}
