//
// Atune - Auto-Tuning of Numerical Applications for GPUs.
//
// - Version 1.0 (April 2011)
//
// - Hans Henrik Brandenborg Sørensen, hhs@imm.dtu.dk
// - GPUlab, DTU Informatics
// - Technical University of Denmark
//
#define m_max 1
#define n_max 177827942
#define m_grid 1
#define n_grid 264
#define m_step (log10(m_max)/m_grid)
#define n_step (log10(n_max)/n_grid)
#define m_idx ((int)(log10(m)/m_step))
#define n_idx ((int)(log10(n)/n_step))

int besttable_sswap[] = { 
1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 34, 9, 44, 1, 33, 5, 9, 9, 3, 14, 9, 6, 14, 14, 13, 8, 13, 9, 14, 14, 10, 12, 9, 22, 10, 10, 10, 14, 12, 10, 21, 11, 23, 13, 13, 10, 10, 10, 10, 10, 10, 10, 2, 22, 2, 4, 2, 10, 10, 10, 10, 2, 21, 10, 10, 21, 2, 4, 6, 4, 21, 11, 11, 10, 5, 10, 11, 1, 21, 11, 31, 21, 10, 21, 21, 22, 22, 21, 4, 11, 10, 21, 6, 4, 4, 21, 7, 10, 31, 10, 10, 10, 10, 10, 10, 31, 10, 10, 32, 10, 21, 10, 10, 41, 36, 36, 41, 44, 41, 45, 18, 40, 43, 47, 37, 18, 32, 11, 24, 16, 16, 27, 12, 30, 19, 26, 38, 23, 17, 20, 17, 28, 42, 43, 46, 44, 21, 41, 41, 48, 41, 31, 41, 41, 21, 31, 41, 31, 41, 41, 41, 31, 31, 41, 31, 31, 41, 21, 31, 41, 21, 31, 39, 21, 21, 31, 21, 21, 10, 10, 10, 31, 10, 39, 10, 10, 10, 39, 10, 10, 31, 10, 10, 15, 10, 10, 15, 10, 10, 10, 10, 10, 10, 10, 15, 15, 15, 29, 25, 29, 31, 29, 31, 29, 29, 29, 29, 29, 29, 29, 29, 31, 31, 49, 49, 49, 49, 49, 49, 49, 49, 49, 49, 49, 49, 49, 49, 49, 49, 49, 49, 49, 49, 35, 35, 35, 35, 
};

int atune_sswap(sswap_arguments)
{
   int bestkernel = besttable_sswap[n_idx];
   switch (bestkernel)
   {
   case 1:
      cublasSswap(n, x_d, 1, y_d, 1);
      break;
   case 2:
      sswap_v1<1, 16, 1, 1>(x_d, y_d, n);
      break;
   case 3:
      sswap_v1<1, 16, 1, 8>(x_d, y_d, n);
      break;
   case 4:
      sswap_v1<1, 32, 1, 1>(x_d, y_d, n);
      break;
   case 5:
      sswap_v1<1, 32, 1, 2>(x_d, y_d, n);
      break;
   case 6:
      sswap_v1<1, 64, 1, 1>(x_d, y_d, n);
      break;
   case 7:
      sswap_v1<1, 64, 1, 2>(x_d, y_d, n);
      break;
   case 8:
      sswap_v1<1, 64, 1, 4>(x_d, y_d, n);
      break;
   case 9:
      sswap_v1<1, 64, 1, 8>(x_d, y_d, n);
      break;
   case 10:
      sswap_v1<1, 128, 1, 1>(x_d, y_d, n);
      break;
   case 11:
      sswap_v1<1, 128, 1, 2>(x_d, y_d, n);
      break;
   case 12:
      sswap_v1<1, 128, 1, 4>(x_d, y_d, n);
      break;
   case 13:
      sswap_v1<1, 128, 1, 8>(x_d, y_d, n);
      break;
   case 14:
      sswap_v1<1, 128, 1, 16>(x_d, y_d, n);
      break;
   case 15:
      sswap_v1<1, 160, 1, 1>(x_d, y_d, n);
      break;
   case 16:
      sswap_v1<1, 160, 1, 4>(x_d, y_d, n);
      break;
   case 17:
      sswap_v1<1, 160, 1, 8>(x_d, y_d, n);
      break;
   case 18:
      sswap_v1<1, 224, 1, 2>(x_d, y_d, n);
      break;
   case 19:
      sswap_v1<1, 224, 1, 4>(x_d, y_d, n);
      break;
   case 20:
      sswap_v1<1, 224, 1, 8>(x_d, y_d, n);
      break;
   case 21:
      sswap_v1<1, 256, 1, 1>(x_d, y_d, n);
      break;
   case 22:
      sswap_v1<1, 256, 1, 2>(x_d, y_d, n);
      break;
   case 23:
      sswap_v1<1, 256, 1, 8>(x_d, y_d, n);
      break;
   case 24:
      sswap_v1<1, 288, 1, 4>(x_d, y_d, n);
      break;
   case 25:
      sswap_v1<1, 320, 1, 1>(x_d, y_d, n);
      break;
   case 26:
      sswap_v1<1, 320, 1, 4>(x_d, y_d, n);
      break;
   case 27:
      sswap_v1<1, 352, 1, 4>(x_d, y_d, n);
      break;
   case 28:
      sswap_v1<1, 384, 1, 8>(x_d, y_d, n);
      break;
   case 29:
      sswap_v1<1, 448, 1, 1>(x_d, y_d, n);
      break;
   case 30:
      sswap_v1<1, 448, 1, 4>(x_d, y_d, n);
      break;
   case 31:
      sswap_v1<1, 512, 1, 1>(x_d, y_d, n);
      break;
   case 32:
      sswap_v1<1, 512, 1, 2>(x_d, y_d, n);
      break;
   case 33:
      sswap_v1<1, 512, 1, 8>(x_d, y_d, n);
      break;
   case 34:
      sswap_v1<1, 512, 1, 16>(x_d, y_d, n);
      break;
   case 35:
      sswap_v1<1, 1024, 1, 4>(x_d, y_d, n);
      break;
   case 36:
      sswap_v2<1, 64, 1, 2>(x_d, y_d, n);
      break;
   case 37:
      sswap_v2<1, 64, 1, 4>(x_d, y_d, n);
      break;
   case 38:
      sswap_v2<1, 64, 1, 8>(x_d, y_d, n);
      break;
   case 39:
      sswap_v2<1, 96, 1, 2>(x_d, y_d, n);
      break;
   case 40:
      sswap_v2<1, 96, 1, 4>(x_d, y_d, n);
      break;
   case 41:
      sswap_v2<1, 128, 1, 2>(x_d, y_d, n);
      break;
   case 42:
      sswap_v2<1, 160, 1, 8>(x_d, y_d, n);
      break;
   case 43:
      sswap_v2<1, 192, 1, 2>(x_d, y_d, n);
      break;
   case 44:
      sswap_v2<1, 256, 1, 2>(x_d, y_d, n);
      break;
   case 45:
      sswap_v2<1, 288, 1, 2>(x_d, y_d, n);
      break;
   case 46:
      sswap_v2<1, 384, 1, 2>(x_d, y_d, n);
      break;
   case 47:
      sswap_v2<1, 416, 1, 2>(x_d, y_d, n);
      break;
   case 48:
      sswap_v2<1, 480, 1, 2>(x_d, y_d, n);
      break;
   case 49:
      sswap_v2<1, 1024, 1, 2>(x_d, y_d, n);
      break;
   default:
      printf("\nsswap.h : bestkernelSwitch : problem size not supported (n = %d, bestkernel = %d).\n", n, bestkernel);
      exit(EXIT_FAILURE);
   }
   return bestkernel;
}
