#pragma once

#include <stdio.h>

#ifdef _DOUBLE
#define float double
#endif

#define cudaAssert( condition ) { if ((condition) != 0) { fprintf( stderr, "\nCUDA FAILURE %s in %s, line %d\n", cudaGetErrorString(condition), __FILE__, __LINE__ ); exit( 1 ); } }

#define H2H cudaMemcpyHostToHost
#define H2D cudaMemcpyHostToDevice
#define D2H cudaMemcpyDeviceToHost
#define D2D cudaMemcpyDeviceToDevice

#define GRIDSIZE_MAX 65536
#define BLOCKSIZE_MIN 32
#define BLOCKSIZE_MAX 128

#define MAXNUMKERNELS 32768
#define MAXNAMELENGTH 128

#define sread_parameters float *x_d, int n, int blockSize_m, int blockSize_n, int workSize_m, int workSize_n, int version
#define sread_call_parameters x_d, n, blockSize_m, blockSize_n, workSize_m, workSize_n, version
#define sread_arguments float *x_d, int n
#define sread_arguments_string "float *x_d, int n"
#define sread_call_arguments x_d, n
#define sread_call_arguments_string "x_d, n"

#define smset_parameters float val, float *y_d, int n, int blockSize_m, int blockSize_n, int workSize_m, int workSize_n, int version
#define smset_call_parameters val, y_d, n, blockSize_m, blockSize_n, workSize_m, workSize_n, version
#define smset_arguments float val, float *y_d, int n
#define smset_arguments_string "float val, float *y_d, int n"
#define smset_call_arguments val, y_d, n
#define smset_call_arguments_string "val, y_d, n"

#define scopy_parameters float *x_d, float *y_d, int n, int blockSize_m, int blockSize_n, int workSize_m, int workSize_n, int version
#define scopy_call_parameters x_d, y_d, n, blockSize_m, blockSize_n, workSize_m, workSize_n, version
#define scopy_arguments float *x_d, float *y_d, int n
#define scopy_arguments_string "float *x_d, float *y_d, int n"
#define scopy_call_arguments x_d, y_d, n
#define scopy_call_arguments_string "x_d, y_d, n"

#define sswap_parameters float *x_d, float *y_d, int n, int blockSize_m, int blockSize_n, int workSize_m, int workSize_n, int version
#define sswap_call_parameters x_d, y_d, n, blockSize_m, blockSize_n, workSize_m, workSize_n, version
#define sswap_arguments float *x_d, float *y_d, int n
#define sswap_arguments_string "float *x_d, float *y_d, int n"
#define sswap_call_arguments x_d, y_d, n
#define sswap_call_arguments_string "x_d, y_d, n"

#define sscal_parameters float alpha, float *x_d, int n, int blockSize_m, int blockSize_n, int workSize_m, int workSize_n, int version
#define sscal_call_parameters alpha, x_d, n, blockSize_m, blockSize_n, workSize_m, workSize_n, version
#define sscal_arguments float alpha, float *x_d, int n
#define sscal_arguments_string "float alpha, float *x_d, int n"
#define sscal_call_arguments alpha, x_d, n
#define sscal_call_arguments_string "alpha, x_d, n"

#define saxpy_parameters float alpha, float *x_d, float *y_d, int n, int blockSize_m, int blockSize_n, int workSize_m, int workSize_n, int version
#define saxpy_call_parameters alpha, x_d, y_d, n, blockSize_m, blockSize_n, workSize_m, workSize_n, version
#define saxpy_arguments float alpha, float *x_d, float *y_d, int n
#define saxpy_arguments_string "float alpha, float *x_d, float *y_d, int n"
#define saxpy_call_arguments alpha, x_d, y_d, n
#define saxpy_call_arguments_string "alpha, x_d, y_d, n"

#define isamax_parameters float *x_d, int *y_d, int n, int blockSize_m, int blockSize_n, int workSize_m, int workSize_n, int version
#define isamax_call_parameters x_d, y_d, n, blockSize_m, blockSize_n, workSize_m, workSize_n, version
#define isamax_arguments float *x_d, int *y_d, int n
#define isamax_arguments_string "float *x_d, int *y_d, int n"
#define isamax_call_arguments x_d, y_d, n
#define isamax_call_arguments_string "x_d, y_d, n"

#define samax_parameters float *x_d, float *y_d, int n, int blockSize_m, int blockSize_n, int workSize_m, int workSize_n, int version
#define samax_call_parameters x_d, y_d, n, blockSize_m, blockSize_n, workSize_m, workSize_n, version
#define samax_arguments float *x_d, float *y_d, int n
#define samax_arguments_string "float *x_d, float *y_d, int n"
#define samax_call_arguments x_d, y_d, n
#define samax_call_arguments_string "x_d, y_d, n"

#define sasum_parameters float *x_d, float *y_d, int n, int blockSize_m, int blockSize_n, int workSize_m, int workSize_n, int version
#define sasum_call_parameters x_d, y_d, n, blockSize_m, blockSize_n, workSize_m, workSize_n, version
#define sasum_arguments float *x_d, float *y_d, int n
#define sasum_arguments_string "float *x_d, float *y_d, int n"
#define sasum_call_arguments x_d, y_d, n
#define sasum_call_arguments_string "x_d, y_d, n"

#define sdot_parameters float *x_d, float *y_d, float *z_d, int n, int blockSize_m, int blockSize_n, int workSize_m, int workSize_n, int version
#define sdot_call_parameters x_d, y_d, z_d, n, blockSize_m, blockSize_n, workSize_m, workSize_n, version
#define sdot_arguments float *x_d, float *y_d, float *z_d, int n
#define sdot_arguments_string "float *x_d, float *y_d, float *z_d, int n"
#define sdot_call_arguments x_d, y_d, z_d, n
#define sdot_call_arguments_string "x_d, y_d, z_d, n"

#define srot_parameters float *x_d, float *y_d, float c, float s, int n, int blockSize_m, int blockSize_n, int workSize_m, int workSize_n, int version
#define srot_call_parameters x_d, y_d, c, s, n, blockSize_m, blockSize_n, workSize_m, workSize_n, version
#define srot_arguments float *x_d, float *y_d, float c, float s, int n
#define srot_arguments_string "float *x_d, float *y_d, float c, float s, int n"
#define srot_call_arguments x_d, y_d, c, s, n
#define srot_call_arguments_string "x_d, y_d, c, s, n"

#define snrm2_parameters float *x_d, float *y_d, int n, int blockSize_m, int blockSize_n, int workSize_m, int workSize_n, int version
#define snrm2_call_parameters x_d, y_d, n, blockSize_m, blockSize_n, workSize_m, workSize_n, version
#define snrm2_arguments float *x_d, float *y_d, int n
#define snrm2_arguments_string "float *x_d, float *y_d, int n"
#define snrm2_call_arguments x_d, y_d, n
#define snrm2_call_arguments_string "x_d, y_d, n"

#define strsv_parameters float *A_d, float *x_d, float *work_d, int m, int lda, int blockSize_m, int blockSize_n, int workSize_m, int workSize_n, int version
#define strsv_call_parameters A_d, x_d, work_d, m, lda, blockSize_m, blockSize_n, workSize_m, workSize_n, version
#define strsv_arguments float *A_d, float *x_d, float *work_d, int m, int lda
#define strsv_arguments_string "float *A_d, float *x_d, float *work_d, int m, int lda"
#define strsv_call_arguments A_d, x_d, work_d, m, lda
#define strsv_call_arguments_string "A_d, x_d, work_d, m, lda"

#define atune_arguments int blockSize_m, int blockSize_n, int workSize_m, int workSize_n, int version
#define atune_call_arguments blockSize_m, blockSize_n, workSize_m, workSize_n, version

#define sgemv_arguments float alpha, float *A_d, int lda, float *x_d, float beta, float *y_d, int m, int n
#define sgemv_arguments_string "float alpha, float *A_d, int lda, float *x_d, float beta, float *y_d, int m, int n"
#define sgemv_call_arguments alpha, A_d, lda, x_d, beta, y_d, m, n
#define sgemv_call_arguments_string "alpha, A_d, lda, x_d, beta, y_d, m, n"

#define sgemv_parameters sgemv_arguments, atune_arguments
#define sgemv_call_parameters sgemv_call_arguments, atune_call_arguments

//#define sgemv_parameters float *A_d, float *x_d, float *y_d, int m, int n, int blockSize_m, int blockSize_n, int workSize_m, int workSize_n, int version
//#define sgemv_call_parameters A_d, x_d, y_d, m, n, blockSize_m, blockSize_n, workSize_m, workSize_n, version

#define dcopy_arguments "(double *x_d, double *y_d, int n)"
#define dcopy_call_arguments "x_d, y_d, n"
#define dnrm2_arguments "(double *x_d, double *y_d, int n)"
#define dnrm2_call_arguments "x_d, y_d, n"
#define dtrsv_arguments "(double *A_d, double *x_d, double *y_d, int m, int lda)"
#define dtrsv_call_arguments "A_d, x_d, y_d, m, lda"
#define dgemv_arguments "(double *A_d, double *x_d, double *y_d, int m, int n)"
#define dgemv_call_arguments "A_d, x_d, y_d, m, n"

#define ssumm_parameters float *A_d, float *y_d, int m, int n, int blockSize_m, int blockSize_n, int workSize_m, int workSize_n, int version
#define sgemm_parameters float *A_d, float *B_d, float *C_d, float *temp_d, int m, int n, int blockSize_m, int blockSize_n, int workSize_m, int workSize_n, int version

#define dnrm2_parameters double *x_d, double *y_d, int n, int blockSize_m, int blockSize_n, int workSize_m, int workSize_n, int version
#define dtrsv_parameters double *A_d, double *x_d, double *y_d, int m, int lda, int blockSize_m, int blockSize_n, int workSize_m, int workSize_n, int version
#define dgemv_parameters double *A_d, double *x_d, double *y_d, int m, int n, int blockSize_m, int blockSize_n, int workSize_m, int workSize_n, int version
#define dsumm_parameters double *A_d, double *y_d, int m, int n, int blockSize_m, int blockSize_n, int workSize_m, int workSize_n, int version
#define dgemm_parameters double *A_d, double *B_d, double *C_d, double *temp_d, int m, int n, int blockSize_m, int blockSize_n, int workSize_m, int workSize_n, int version

#define MAX_SMEM_PER_BLOCK 16384
#define MIN_BLOCKS_PER_MP 1
