#include "defines.h"
#include "smset_kernels.cu"
#include <cublas.h>

#if not defined (_TESLA) && not defined (_FERMI)
#error Requires an architechture flag (either _TESLA or _FERMI) to compile.
#endif

#define switchParameters val, y_d
#include "level1_templatemacros.h"

#ifdef _ATUNE

// When building atune
#define arguments smset_parameters
#define call_arguments smset_call_parameters
#define switchMacro workSizeSwitchSmallPow_n
#define templatedec

#else

// When building library
#define BLOCKSIZE_n BLOCKSIZE
#define blockSize_n BLOCKSIZE_n
#define workSize_n WORKSIZE_n
#define arguments smset_arguments
#define call_arguments smset_call_arguments
#define switchMacro call_kernel
#define templatedec template<int BLOCKSIZE_m, int BLOCKSIZE_n, int WORKSIZE_m, int WORKSIZE_n>

#endif

templatedec
static int smset_v1(arguments)
{		
	//
	// Generic version
	//
	// Determine size of grid
	int gridSize_n = (n + blockSize_n * workSize_n - 1) / (blockSize_n * workSize_n);
	if (gridSize_n > 65535) return -1; 
	
	// Sizes of the tiled part
	int ntile = (n / (blockSize_n * workSize_n)) * (blockSize_n * workSize_n);
	
	// Kernel specifications
	dim3 dimGrid(gridSize_n);
	dim3 dimBlock(blockSize_n);
	
	// Call nested switch macros for all templated values
	//printf("\nSmset-%d: n=%d grid_n=%d ntile=%d block_m=%d block_n=%d work_m=%d work_n=%d\n", version, n, gridSize_n, ntile, blockSize_m, blockSize_n, workSize_m, workSize_n);
	switchMacro(smset_kernel1, BLOCKSIZE, 1, workSize_n);
	return 0;
}


#ifdef _ATUNE

// When building atune
template <int BLOCKSIZE_n>
int smset(smset_parameters);

template <>
int smset<BLOCKSIZE>(smset_parameters)
{
	if (version == 1)
	{
		return smset_v1(smset_call_parameters);
	}

	// return "not run" signal to auto-tuner
	return -1;
}

#else

//When building library
#include "atune_headers/smset.h"
int glas_smset(int n, float val, float *y_d) 
{
	return atune_smset(smset_call_arguments);
}

#endif
