#include "defines.h"
#include "sscal_kernels.cu"
#include <cublas.h>

#if not defined (_TESLA) && not defined (_FERMI)
#error Requires an architechture flag (either _TESLA or _FERMI) to compile.
#endif

#define switchParameters alpha, x_d
#include "level1_templatemacros.h"

#ifdef _ATUNE

// When building atune
#define arguments sscal_parameters
#define call_arguments sscal_call_parameters
#define switchMacro workSizeSwitchSmallPow_n
#define templatedec

#else

// When building library
#define BLOCKSIZE BLOCKSIZE_n
#define blockSize_n BLOCKSIZE_n
#define workSize_n WORKSIZE_n
#define arguments sscal_arguments
#define call_arguments sscal_call_arguments
#define switchMacro call_kernel
#define templatedec template<int BLOCKSIZE_m, int BLOCKSIZE_n, int WORKSIZE_m, int WORKSIZE_n>

#endif

templatedec
static int sscal_v1(arguments)
{		
	//
	// Generic version
	//

	// Determine size of grid (make sure that we are within grid dimension limits)
	int gridSize_n = (n + blockSize_n * workSize_n - 1) / (blockSize_n * workSize_n);
	if (gridSize_n > 65535) return -1; 
	
	// Sizes of the tiled part
	int ntile = (n / (blockSize_n * workSize_n)) * (blockSize_n * workSize_n);
	
	// Kernel specifications
	dim3 dimGrid(gridSize_n);
	dim3 dimBlock(blockSize_n);
	
	// Call nested switch macros for all templated values
	//printf("\nSscal-%d: n=%d grid_n=%d ntile=%d block_m=%d block_n=%d work_m=%d work_n=%d\n", version, n, gridSize_n, ntile, blockSize_m, blockSize_n, workSize_m, workSize_n);
	switchMacro(sscal_kernel1, BLOCKSIZE, 1, workSize_n);
	return 0;
}

templatedec
static int sscal_v2(arguments)
{		
	//
	// Generic version using doubles
	//

	// Determine size of grid
	int gridSize_n = (n + blockSize_n * workSize_n - 1) / (blockSize_n * workSize_n);
	if (gridSize_n > 65535) return -1; 
	
	// Sizes of the tiled part
	int ntile = (n / (blockSize_n * workSize_n)) * (blockSize_n * workSize_n);
	
	// Kernel specifications
	dim3 dimGrid(gridSize_n);
	dim3 dimBlock(blockSize_n);
	
	// Call nested switch macros for all templated values
	//printf("\nSscal-%d: n=%d grid_n=%d ntile=%d block_m=%d block_n=%d work_m=%d work_n=%d\n", version, n, gridSize_n, ntile, blockSize_m, blockSize_n, workSize_m, workSize_n);
	switchMacro(sscal_kernel2, BLOCKSIZE, 1, workSize_n);
	return 0;
}

#ifdef _ATUNE

// When building atune
template <int BLOCKSIZE_n>
int sscal(sscal_parameters);

template <>
int sscal<BLOCKSIZE>(sscal_parameters)
{
	if (version == 1)
	{
		return sscal_v1(sscal_call_parameters);
	}
	else if (version == 2 && !(workSize_n & 1))
	{
		return sscal_v2(sscal_call_parameters);
	}

	// return "not run" signal to auto-tuner
	return -1;
}

#else

//When building library
#include "atune_headers/sscal.h"
int glas_sscal(int n, float alpha, float *x_d, int incx) 
{

	if (incx == 1)
	{
		return atune_sscal(sscal_call_arguments);
	}
	else 
	{
		printf("Call to sscal: parameter 4 has an unsupported value (%d)!\n", incx); exit(EXIT_FAILURE);
	}
}

#endif
