#pragma once

#define call_kernel1(kernel, WORKSIZE_m, BLOCKSIZE_m) \
	kernel<BLOCKSIZE_m, BLOCKSIZE_n, WORKSIZE_m><<< dimGrid, dimBlock >>>(A_d, lda, x_d, Ainv_d, m, mtile, ntile);

#define workSizeSwitch1Pow_n(kernel, WORKSIZE_m, workSize_n) \
	switch (workSize_n)\
	{\
	case 16:\
		kernel<16, BLOCKSIZE / 16, WORKSIZE_m><<< dimGrid, dimBlock >>>(A_d, lda, x_d, Ainv_d, m, mtile, ntile); break; \
	case 32:\
		kernel<32, BLOCKSIZE / 32 + (BLOCKSIZE / 32 == 0), WORKSIZE_m><<< dimGrid, dimBlock >>>(A_d, lda, x_d, Ainv_d, m, mtile, ntile); break; \
	default:\
		printf("\n workSizeSwitch1Pow_n : workSize not supported. [workSize_m = %i workSize_n = %i (workSize = %i)]\n", WORKSIZE_m, workSize_n, WORKSIZE_m * workSize_n); \
		exit(EXIT_FAILURE);\
	}

#define workSizeSwitch1Pow_m_Pow_n(kernel, workSize_m, workSize_n) \
	switch (workSize_m)\
	{\
	case 1:\
		workSizeSwitch1Pow_n(kernel, 1, workSize_n); break; \
	case 2:\
		workSizeSwitch1Pow_n(kernel, 2, workSize_n); break; \
	case 4:\
		workSizeSwitch1Pow_n(kernel, 4, workSize_n); break; \
	case 8:\
		workSizeSwitch1Pow_n(kernel, 8, workSize_n); break; \
	case 16:\
		workSizeSwitch1Pow_n(kernel, 16, workSize_n); break; \
	default:\
		printf("\n workSizeSwitch1Pow_m_Pow_n : workSize not supported. [workSize_m = %i workSize_n = %i (workSize = %i)]\n", workSize_m, workSize_n, workSize_m * workSize_n);\
		exit(EXIT_FAILURE);\
	}

#define call_kernel2(kernel, size, WORKSIZE_n) \
	switch (size)\
	{\
	case 16:\
		kernel<16, WORKSIZE_n, 1><<< dimGrid, dimBlock >>>(A_d + size, lda, Ainv_d, tileSize, Ainv_d + size * tileSize, tileSize / size2, m, mtile, ntile); \
		kernel<16, WORKSIZE_n, 2><<< dimGrid, dimBlock >>>(Ainv_d + size + size * tileSize, tileSize, Ainv_d + size * tileSize, tileSize, Ainv_d + size, tileSize / size2, m, mtile, ntile); break; \
	case 32:\
		kernel<32, WORKSIZE_n, 1><<< dimGrid, dimBlock >>>(A_d + size, lda, Ainv_d, tileSize, Ainv_d + size * tileSize, tileSize / size2, m, mtile, ntile); \
		kernel<32, WORKSIZE_n, 2><<< dimGrid, dimBlock >>>(Ainv_d + size + size * tileSize, tileSize, Ainv_d + size * tileSize, tileSize, Ainv_d + size, tileSize / size2, m, mtile, ntile); break; \
		workSizeSwitch2Pow_n(kernel, 32, WORKSIZE_n); break; \
	case 64:\
		kernel<64, WORKSIZE_n, 1><<< dimGrid, dimBlock >>>(A_d + size, lda, Ainv_d, tileSize, Ainv_d + size * tileSize, tileSize / size2, m, mtile, ntile); \
		kernel<64, WORKSIZE_n, 2><<< dimGrid, dimBlock >>>(Ainv_d + size + size * tileSize, tileSize, Ainv_d + size * tileSize, tileSize, Ainv_d + size, tileSize / size2, m, mtile, ntile); break; \
	case 128:\
		kernel<128, WORKSIZE_n, 1><<< dimGrid, dimBlock >>>(A_d + size, lda, Ainv_d, tileSize, Ainv_d + size * tileSize, tileSize / size2, m, mtile, ntile); \
		kernel<128, WORKSIZE_n, 2><<< dimGrid, dimBlock >>>(Ainv_d + size + size * tileSize, tileSize, Ainv_d + size * tileSize, tileSize, Ainv_d + size, tileSize / size2, m, mtile, ntile); break; \
	case 256:\
		kernel<256, WORKSIZE_n, 1><<< dimGrid, dimBlock >>>(A_d + size, lda, Ainv_d, tileSize, Ainv_d + size * tileSize, tileSize / size2, m, mtile, ntile); \
		kernel<256, WORKSIZE_n, 2><<< dimGrid, dimBlock >>>(Ainv_d + size + size * tileSize, tileSize, Ainv_d + size * tileSize, tileSize, Ainv_d + size, tileSize / size2, m, mtile, ntile); break; \
	default:\
		printf("\n strsv::call_kernel2 : workSize not supported. [workSize_m = %i workSize_n = %i (workSize = %i)]\n", workSize_m, workSize_n, workSize_m * workSize_n);\
		exit(EXIT_FAILURE);\
	}

#define workSizeSwitch2Pow_n(kernel, WORKSIZE_m, workSize_n) \
	switch (workSize_n)\
	{\
	case 1:\
		kernel<WORKSIZE_m, 1, 1><<< dimGrid, dimBlock >>>(A_d + size, lda, Ainv_d, tileSize, Ainv_d + size * tileSize, tileSize / size2, m, mtile, ntile); \
		kernel<WORKSIZE_m, 1, 2><<< dimGrid, dimBlock >>>(Ainv_d + size + size * tileSize, tileSize, Ainv_d + size * tileSize, tileSize, Ainv_d + size, tileSize / size2, m, mtile, ntile); break; \
	case 2:\
		kernel<WORKSIZE_m, 2, 1><<< dimGrid, dimBlock >>>(A_d + size, lda, Ainv_d, tileSize, Ainv_d + size * tileSize, tileSize / size2, m, mtile, ntile); \
		kernel<WORKSIZE_m, 2, 2><<< dimGrid, dimBlock >>>(Ainv_d + size + size * tileSize, tileSize, Ainv_d + size * tileSize, tileSize, Ainv_d + size, tileSize / size2, m, mtile, ntile); break; \
	case 4:\
		kernel<WORKSIZE_m, 4, 1><<< dimGrid, dimBlock >>>(A_d + size, lda, Ainv_d, tileSize, Ainv_d + size * tileSize, tileSize / size2, m, mtile, ntile); \
		kernel<WORKSIZE_m, 4, 2><<< dimGrid, dimBlock >>>(Ainv_d + size + size * tileSize, tileSize, Ainv_d + size * tileSize, tileSize, Ainv_d + size, tileSize / size2, m, mtile, ntile); break; \
	case 8:\
		kernel<WORKSIZE_m, 8, 1><<< dimGrid, dimBlock >>>(A_d + size, lda, Ainv_d, tileSize, Ainv_d + size * tileSize, tileSize / size2, m, mtile, ntile); \
		kernel<WORKSIZE_m, 8, 2><<< dimGrid, dimBlock >>>(Ainv_d + size + size * tileSize, tileSize, Ainv_d + size * tileSize, tileSize, Ainv_d + size, tileSize / size2, m, mtile, ntile); break; \
	case 16:\
		kernel<WORKSIZE_m, 16, 1><<< dimGrid, dimBlock >>>(A_d + size, lda, Ainv_d, tileSize, Ainv_d + size * tileSize, tileSize / size2, m, mtile, ntile); \
		kernel<WORKSIZE_m, 16, 2><<< dimGrid, dimBlock >>>(Ainv_d + size + size * tileSize, tileSize, Ainv_d + size * tileSize, tileSize, Ainv_d + size, tileSize / size2, m, mtile, ntile); break; \
	default:\
		printf("\n workSizeSwitch2Pow_n : workSize not supported. [workSize_m = %i workSize_n = %i (workSize = %i)]\n", WORKSIZE_m, workSize_n, WORKSIZE_m * workSize_n); \
		exit(EXIT_FAILURE);\
	}

#define workSizeSwitch2Pow_m_Pow_n(kernel, workSize_m, workSize_n) \
	switch (workSize_m)\
	{\
	case 16:\
		workSizeSwitch2Pow_n(kernel, 16, workSize_n); break; \
	case 32:\
		workSizeSwitch2Pow_n(kernel, 32, workSize_n); break; \
	case 64:\
		workSizeSwitch2Pow_n(kernel, 64, workSize_n); break; \
	case 128:\
		workSizeSwitch2Pow_n(kernel, 128, workSize_n); break; \
	case 256:\
		workSizeSwitch2Pow_n(kernel, 256, workSize_n); break; \
	default:\
		printf("\n workSizeSwitch2Pow_m_Pow_n : workSize not supported. [workSize_m = %i workSize_n = %i (workSize = %i)]\n", workSize_m, workSize_n, workSize_m * workSize_n);\
		exit(EXIT_FAILURE);\
	}

